unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeeGDIPlus, ExtCtrls, TeeProcs, TeEngine, Chart, Series, TeeTools;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure Chart1Zoom(Sender: TObject);
    procedure Chart1UndoZoom(Sender: TObject);
    procedure Chart1Scroll(Sender: TObject);
    procedure Chart1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    procedure DragMarksOnDragged(Sender:TDragMarksTool; Index:Integer;
                                    Button:TMouseButton; Shift: TShiftState;
                                    X,Y: Integer);
    procedure PlaceMarks;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

var Chart1: TChart;
    XOffset, YOffset: Array of Integer;
    MouseDownPos: TPoint;

procedure TForm1.FormCreate(Sender: TObject);
var i: Integer;
begin
  Chart1:=TChart.Create(Self);
  Chart1.Parent:=Self;
  Chart1.Align:=alClient;

  Chart1.Legend.Visible:=false;
  Chart1.View3D:=false;

  with Chart1.AddSeries(TBarSeries) do
  begin
    FillSampleValues;
    ColorEachPoint:=true;
    Marks.Arrow.Color:=clRed;
  end;

  SetLength(XOffset, Chart1[0].Count);
  SetLength(YOffset, Chart1[0].Count);

  for i:=0 to High(XOffset) do
  begin
    XOffset[i]:=0;
    YOffset[i]:=0;
  end;

  with Chart1.Tools.Add(TDragMarksTool) as TDragMarksTool do
  begin
    Series:=Chart1[0];
    OnDraggedMark:=DragMarksOnDragged;
  end;

  Chart1.OnZoom:=Chart1Zoom;
  Chart1.OnUndoZoom:=Chart1UndoZoom;
  Chart1.OnScroll:=Chart1Scroll;
  Chart1.OnMouseDown:=Chart1MouseDown;
end;

procedure TForm1.DragMarksOnDragged(Sender:TDragMarksTool; Index:Integer;
                                    Button:TMouseButton; Shift: TShiftState;
                                    X,Y: Integer);
begin
  if Assigned(Sender.Series) then
  begin
    if (Index>-1) and (Index<Length(XOffset)) then
    begin
      XOffset[Index]:=XOffset[Index]+X-MouseDownPos.X;
      YOffset[Index]:=YOffset[Index]+Y-MouseDownPos.Y;
    end;
  end;
end;

procedure TForm1.PlaceMarks;
var s, i, tmpWidth, tmpHeight, tmpX, tmpY: Integer;
begin
  Chart1.Draw;

  for s:=0 to Chart1.SeriesCount-1 do
    if Chart1[s] is TBarSeries then
      with Chart1[s] as TBarSeries do
      begin
        for i:=0 to Count-1 do
          with Marks.Positions[i] do
          begin
            tmpX:=CalcXPos(i);
            tmpY:=CalcYPos(i);

            tmpWidth:=Canvas.TextWidth(ValueMarkText[i])+12;
            tmpHeight:=Canvas.TextHeight(ValueMarkText[i])+2;
            Custom:=true;
            LeftTop.X:=tmpX + BarWidth div 2 - tmpWidth div 2 + XOffset[i];
            LeftTop.Y:=tmpY - Marks.ArrowLength - tmpHeight + YOffset[i];
            ArrowFrom:=Point(tmpX + BarWidth div 2, tmpY);
            ArrowTo:=Point(LeftTop.X + tmpWidth div 2, LeftTop.Y);
          end;
      end;
end;

procedure TForm1.Chart1Scroll(Sender: TObject);
begin
  PlaceMarks;
end;

procedure TForm1.Chart1Zoom(Sender: TObject);
begin
  PlaceMarks;
end;

procedure TForm1.Chart1UndoZoom(Sender: TObject);
begin
  PlaceMarks;
end;

procedure TForm1.Chart1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  MouseDownPos.X:=X;
  MouseDownPos.Y:=Y;
end;

end.
